<?php
/*
 * This file is part of Robot-in
 *
 * Copyright (C) 2006-2012 Hunglead,Inc. All Rights Reserved.
 *
 * http://www.hunglead.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/**
 * 受注情報用データアクセスオブジェクト
 * @author hunglead
 */
class OrderDao {

	/**
	 * 受注情報を取得
	 * @param array $id アカウント情報
	 * @param number $status 取得先ステータス
	 * @param string $fromDate 検索期間開始日
	 * @param string $toDate 検索期間終了日
	 */
	public function getOrderInfos($id, $status, $fromDate, $toDate) {
		$this->checkId($id);

		$status = $status ? $status : 1;

		$where = "del_flg = 0";
		$whereValues = array();

		$where .= " AND status = ?";
		$whereValues[] = $status;

		if ($fromDate) {
			$where .= " AND create_date >= ?";
			$whereValues[] = $fromDate;
		}

		if ($toDate) {
			$toDate = DateUtil::createDateTime($toDate);
			$toDate->modify("+1 day");
			$where .= " AND create_date <= ?";
			$whereValues[] = $toDate->format("Y/m/d");
		}

		$masterData = new SC_DB_MasterData_Ex();
		$options = array(
			"arrJob" => $masterData->getMasterData('mtb_job'),
			"arrPref" => $masterData->getMasterData('mtb_pref'),
			"arrStatus" => $masterData->getMasterData('mtb_order_status'),
			"arrDevice" => $masterData->getMasterData('mtb_device_type'),
			"arrDeliv" => SC_Helper_DB_Ex::sfGetIDValueList('dtb_deliv', 'deliv_id', 'name')
		);

		$objQuery =& SC_Query_Ex::getSingletonInstance();
		$purchase = new SC_Helper_Purchase_Ex();
		$results = array();

		foreach ($objQuery->select('order_id', 'dtb_order', $where, $whereValues) as $row) {

			$srcOrder = $purchase->getOrder($row["order_id"]);
			if ($srcOrder["del_flag"] == 1) {
				continue;
			}

			$srcShippings = $purchase->getShippings($row["order_id"]);
			
			//ShippingItem補足
			if($this->isSetShippingItems($srcShippings) == false){
				foreach($srcShippings as &$shippingInfo){
					if($shippingInfo['del_flag'] == 1){
						continue;
					}
					
					$shippingInfo['shipment_item'] = $purchase->getOrderDetail($row["order_id"], false);
					break;
				}
			}
			
			$order = $this->exchangeOrder($srcOrder, $srcShippings, $options);
			$order["unique_key"] = $order["order"]["order_date"].$order["order"]["order_code"];
			$results[] = $order;
		}

		return $results;
	}
	
	/**
	 * shipment_item設定されたかをチェック
	 * @param array $srcShippings
	 */
	public function isSetShippingItems($srcShippings){
		
		$hasShippingItems = false;
		
		foreach($srcShippings as $shoppingId => $shoppingInfo){
			if(is_array($shoppingInfo['shipment_item']) && count($shoppingInfo['shipment_item']) > 0){	
				$hasShippingItems = true;
				break;
			}
		}
		
		return $hasShippingItems;
	}
	
	/**
	 * 受注情報を取得
	 * @param array $id アカウント情報
	 * @param int $orderId 受注ID
	 */
	public function getOrderInfo($id, $orderId) {
		$this->checkId($id);

		$purchase = new SC_Helper_Purchase_Ex();

		$srcOrder = $purchase->getOrder($orderId);
		if ($srcOrder["del_flag"] == 1) {
			return;
		}

		$masterData = new SC_DB_MasterData_Ex();
		$options = array(
				"arrJob" => $masterData->getMasterData('mtb_job'),
				"arrPref" => $masterData->getMasterData('mtb_pref'),
				"arrStatus" => $masterData->getMasterData('mtb_order_status'),
				"arrDevice" => $masterData->getMasterData('mtb_device_type'),
				"arrDeliv" => SC_Helper_DB_Ex::sfGetIDValueList('dtb_deliv', 'deliv_id', 'name')
		);

		$srcShippings = $purchase->getShippings($orderId);
		$order = $this->exchangeOrder($srcOrder, $srcShippings, $options);
		$order["unique_key"] = $order["order"]["order_date"].$order["order"]["order_code"];

		return $order;

	}

	/**
	 * EC-CUBE形式の受注情報をHARC形式の受注情報に変換
	 * @param array $srcOrder EC-CUBE形式受注情報
	 * @param array $srcShippings EC-CUBE形式送り先情報
	 * @param array $options EC-CUBEマスター情報
	 * @return array HARC形式受注情報
	 */
	private function exchangeOrder($srcOrder, $srcShippings, $options) {

		$arrJob = $options["arrJob"];
		$arrPref = $options["arrPref"];
		$arrStatus = $options["arrStatus"];
		$arrDevice = $options["arrDevice"];
		$arrDeliv = $options["arrDeliv"];

		$destOrder = array();
		$destOrder["order_code"] = $srcOrder["order_id"];
		$destOrder["order_date"] = $srcOrder["create_date"];
		$destOrder["order_status"] = $arrStatus[$srcOrder["status"]];
		$destOrder["payment_method"] = $srcOrder["payment_method"];
		$destOrder["payment_date"] = $srcOrder["payment_date"];
		$destOrder["order_price"] = $srcOrder["payment_total"];
		$destOrder["delivery_cost"] = $srcOrder["deliv_fee"];
		$destOrder["commission"] = $srcOrder["charge"];
		$destOrder["discount"] = $srcOrder["discount"];
		$destOrder["use_point"] = $srcOrder["use_point"];
		$destOrder["remarks"] = $srcOrder["message"];

		$destCustomer = array();
		$destCustomer["name"] = $srcOrder["order_name01"]." ".$srcOrder["order_name02"];
		$destCustomer["kana"] = $srcOrder["order_kana01"]." ".$srcOrder["order_kana02"];
		$destCustomer["zipcode"] = $srcOrder["order_zip01"]."-".$srcOrder["order_zip02"];
		$destCustomer["address"] = $arrPref[$srcOrder["order_pref"]].$srcOrder["order_addr01"].$srcOrder["order_addr02"];
		$destCustomer["phone"] = $srcOrder["order_tel01"]."-".$srcOrder["order_tel02"]."-".$srcOrder["order_tel03"];
		$destCustomer["email"] = $srcOrder["order_email"];
		$destCustomer["company"] = $srcOrder["order_company_name"];

		$destOther = array();
		if ($srcOrder["order_fax01"] && $srcOrder["order_fax02"] && $srcOrder["order_fax03"]) {
			$destOther["order_fax"] = $srcOrder["order_fax01"]."-".$srcOrder["order_fax02"]."-".$srcOrder["order_fax03"];
		} else {
			$destOther["order_fax"] = null;
		}

		$destOther["order_sex"] = $srcOrder["order_sex"] ? $srcOrder["order_sex"] == 1 ? "男性" : "女性" : "不明";
		$destOther["order_birth"] = $srcOrder["order_birth"];
		$destOther["order_job"] = $arrJob[$srcOrder["order_job"]];
		$destOther["device_type"] = $arrDevice[$srcOrder["device_type_id"]];
		$destOther["update_date"] = $srcOrder["update_date"];
		$destOther["commit_date"] = $srcOrder["commit_date"];
		$destOther["memo01"] = $srcOrder["memo01"];
		$destOther["memo02"] = $srcOrder["memo02"];
		$destOther["memo03"] = $srcOrder["memo03"];
		$destOther["memo04"] = $srcOrder["memo04"];
		$destOther["memo05"] = $srcOrder["memo05"];
		$destOther["memo06"] = $srcOrder["memo06"];
		$destOther["memo07"] = $srcOrder["memo07"];
		$destOther["memo08"] = $srcOrder["memo08"];
		$destOther["memo09"] = $srcOrder["memo09"];
		$destOther["memo10"] = $srcOrder["memo10"];

		$delivMethod = $arrDeliv[$srcOrder["deliv_id"]];

		$destDetails = array();

		foreach ($srcShippings as $shipping) {

			if ($shipping["del_flag"] == 1) {
				continue;
			}

			foreach ($shipping["shipment_item"] as $shipmentItem) {

				$destDetail = array();
				$destDetail["item_code"] = $shipmentItem["product_code"];
				$destDetail["item_name"] = $shipmentItem["product_name"];
				if ($shipmentItem["classcategory_name1"] && $shipmentItem["classcategory_name2"]) {
					$destDetail["item_select"] = $shipmentItem["classcategory_name1"]."/".$shipmentItem["classcategory_name2"];
				} else if ($shipmentItem["classcategory_name1"]) {
					$destDetail["item_select"] =  $shipmentItem["classcategory_name1"];
				} else {
					$destDetail["item_select"] = null;
				}

				$destDetail["unit_price"] = $shipmentItem["price"];
				$destDetail["quantity"] = $shipmentItem["quantity"];

				$destDelivery = array();
				$destDelivery["delivery_method"] = $delivMethod;
				$destDelivery["delivery_hope_date"] = $shipping["shipping_date"];
				$destDelivery["delivery_hope_time"] = $shipping["shipping_time"];
				$destDelivery["delivery_memo"] = null;

				$destShip = array();
				$destShip["name"] = $shipping["shipping_name01"]." ".$shipping["shipping_name02"];
				$destShip["kana"] = $shipping["shipping_kana01"]." ".$shipping["shipping_kana02"];
				$destShip["zipcode"] = $shipping["shipping_zip01"]."-".$shipping["shipping_zip02"];
				$destShip["address"] = $arrPref[$shipping["shipping_pref"]].$shipping["shipping_addr01"].$shipping["shipping_addr02"];
				$destShip["phone"] = $shipping["shipping_tel01"]."-".$shipping["shipping_tel02"]."-".$shipping["shipping_tel03"];
				$destShip["company"] = $shipping["shipping_company_name"];

				$destDetail["delivery"] = $destDelivery;
				$destDetail["ship"] = $destShip;

				$destDetails[] = $destDetail;
			}
		}

		$destOrder["order_tax"] = $srcOrder["tax"];

		$destOrder["other"] = $destOther;

		$item = array();
		$item["order"] = $destOrder;
		$item["customer"] = $destCustomer;
		$item["details"] = $destDetails;

		return $item;
	}

	/**
	 * ステータスを変更
	 * @param array $id アカウント情報
	 * @param array $targetOrders 対象受注情報
	 * @param number $status 移動先ステータス
	 * @return array ステータス変更結果
	 */
	public function changeStatus($id, $targetOrders, $status) {
		$this->checkId($id);

		$objPurchase = new SC_Helper_Purchase_Ex();
		$objQuery =& SC_Query_Ex::getSingletonInstance();
		$objQuery->begin();

		$results = array();

		foreach ($targetOrders as $order) {

			$orderId = $order["orderCode"];

			if (!$orderId) {
				continue;
			}

			if (!preg_match("/^[0-9]+$/u", $orderId)) {
				$results[$orderId] = "Invalid OrderCode.";
				continue;
			}

			$order = $objPurchase->getOrder($orderId);
			if ($order) {

				if ($order["status"] == $status) {
					$results[$orderId] = "Needless To Move.";
				} else {

					$objPurchase->sfUpdateOrderStatus($orderId, $status);
					$results[$orderId] = "Success.";
				}

			} else {
				$results[$orderId] = "Not Found.";
			}

		}

		$objQuery->commit();

		return $results;
	}

	/**
	 * アカウント情報を検証
	 * @param array $id アカウント情報
	 * @throws Exception
	 */
	private function checkId($id) {
		$account = $id["account"];
		$password = $id["password"];

		if (!$account || !$password) {
			throw new Exception("Invalid Account.");
		}

		$objQuery =& SC_Query_Ex::getSingletonInstance();
		$cols = 'password, salt';
		$table = 'dtb_member';
		$where = 'login_id = ? AND del_flg <> 1 AND work = 1';
		$arrData = $objQuery->getRow($cols, $table, $where, array($account));
		if (SC_Utils_Ex::isBlank($arrData)) {
			throw new Exception("Invalid Account.");
		}

		if (!SC_Utils_Ex::sfIsMatchHashPassword($password, $arrData['password'], $arrData['salt'])) {
			throw new Exception("Invalid Account.");
		}
	}
}