<?php
/*
* This file is part of Robot-in
*
* Copyright (C) 2006-2012 Hunglead,Inc. All Rights Reserved.
*
* http://www.hunglead.com/
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/**
 * 商品情報用データアクセスオブジェクト
 * @author hunglead
 *
 */
class ProductDao {

	/**
	 * EXISTS文を実行する.
	 *
	 * @param object $objQuery クエリオブジェクト
	 * @param string $table テーブル名
	 * @param string $where where句
	 * @param array $arrWhereVal プレースホルダ
	 * @return boolean 有無
	 */
	function exists($objQuery, $table, $where = '', $arrWhereVal = array()) {
		$sql_inner = $objQuery->getSql('*', $table, $where, $arrWhereVal);
		$sql = "SELECT CASE WHEN EXISTS($sql_inner) THEN 1 ELSE 0 END";
		$res = $objQuery->getOne($sql, $arrWhereVal);
		return (bool)$res;
	}

    /**
     * 商品情報を更新
     * @param array $id アカウント情報
     * @param array $products 商品情報
     */
    public function update($id, $products) {
        $this->checkId($id);

        if (isset($products)) {

            $objQuery = new SC_Query();

            $table = "dtb_products_class";
            $where = "product_code = ?";

            $cnt = 0;

            foreach($products as $key => $val) {

                $productCode = empty($val["product_code"]) ? 0 : $val["product_code"];

                $arrval = array();
                $arrval[] = $productCode;

                if ($this->exists($objQuery, $table, $where, $arrval) === true) {

                    $stock = $val["stock"];
                    $stock = $stock == -1 ? null : $stock;
                    $price = $val["price"];
                    $price = $price == -1 ? null : $price;
                    //$label = $val["label"];

                    $sqlval = array();
                    if (isset($stock) && $stock != "") {
                        $sqlval["stock"] = $stock;
                    }
                    if (isset($price) && $price != "") {
                        $sqlval["price02"] = $price;
                    }

                    if (empty($sqlval)) {
                        continue;
                    }

                    $objQuery->update($table, $sqlval, $where, $arrval);

                } else {
                    $cnt++;
                    if ($cnt == 1) {
                        $mes = "NG\n";
                    }

                    $mes .= "no_product(product_code=". $productCode. ")\n";
                }
            }

            if ($cnt == 0) {
                $mes = "OK";
            }

        } else {
            $mes = "NG\n". "no_query";
        }

        return $mes;
    }

    /**
    * アカウント情報を検証
    * @param array $id アカウント情報
    * @throws Exception
    */
    private function checkId($id) {
        $account = $id["account"];
        $password = $id["password"];

        if (!$account || !$password) {
            throw new Exception("Invalid Account.");
        }

        $objQuery = new SC_Query();
        $cols = 'password, salt';
        $table = 'dtb_member';
        $where = 'login_id = ? AND del_flg <> 1 AND work = 1';
        $arrData = $objQuery->getRow($cols, $table, $where, array($account));
        if (SC_Utils_Ex::isBlank($arrData)) {
            throw new Exception("Invalid Account.");
        }

        if (!SC_Utils_Ex::sfIsMatchHashPassword($password, $arrData['password'], $arrData['salt'])) {
            throw new Exception("Invalid Account.");
        }
    }
}