<?php
/*
 * This file is part of Robot-in
 *
 * Copyright (C) 2006-2012 Hunglead,Inc. All Rights Reserved.
 *
 * http://www.hunglead.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

require DATA_REALDIR . 'module/fpdf/fpdf.php';
require DATA_REALDIR . 'module/fpdi/japanese.php';
require_once HUNGLEAD_PLUGIN_DIR. "harc/ApiUtil.php";

/**
 * PDF出力用クラス
 * @author hunglead
 */
class CustomFpdf extends PDF_Japanese {

	function __construct($download, $title, $tpl_pdf = 'nouhinsyo1.pdf') {
        $this->FPDF();
        // デフォルトの設定
        $this->tpl_pdf = HUNGLEAD_PLUGIN_DIR."utils/".$tpl_pdf;  // テンプレートファイル
        $this->pdf_download = $download;      // PDFのダウンロード形式（0:表示、1:ダウンロード）
        $this->tpl_title = $title;
        $this->tpl_dispmode = 'real';      // 表示モード
        $masterData = new SC_DB_MasterData_Ex();
        $this->arrPref = $masterData->getMasterData('mtb_pref');
        $this->width_cell = array(110.3,12,21.7,24.5);

        $this->label_cell[] = $this->lfConvSjis('商品名 / 商品コード / [ 規格 ]');
        $this->label_cell[] = $this->lfConvSjis('数量');
        $this->label_cell[] = $this->lfConvSjis('単価');
        $this->label_cell[] = $this->lfConvSjis('金額');

        $this->arrMessage = array(
            'このたびはお買上げいただきありがとうございます。',
            '下記の内容にて納品させていただきます。',
            'ご確認くださいますよう、お願いいたします。'
        );

        $this->lfAddGothicFont();

        // SJISフォント
        $this->AddSJISFont();
        $this->SetFont('SJIS');

        //ページ総数取得
        $this->AliasNbPages();

        // マージン設定
        $this->SetMargins(15, 20);

        // PDFを読み込んでページ数を取得
        $pageno = $this->setSourceFile($this->tpl_pdf);
    }

    function loadCompanyData() {
    	$this->companyInfo = ApiUtil::getCompanyInfo();
    }

    function setData($arrData) {
        $this->arrData = $arrData;

        // ページ番号よりIDを取得
        $tplidx = $this->ImportPage(1);

        // ページを追加（新規）
        $this->AddPage();

        //表示倍率(100%)
        $this->SetDisplayMode($this->tpl_dispmode);

        if (SC_Utils_Ex::sfIsInt($arrData['order_id'])) {
            $this->disp_mode = true;
            $order_id = $arrData['order_id'];
        }

        // テンプレート内容の位置、幅を調整 ※useTemplateに引数を与えなければ100%表示がデフォルト
        $this->useTemplate($tplidx);

        $this->lfGetData($this->arrData['order_id']);

        $this->setShopData();
        $this->setMessageData();
        $this->setOrderData();
        $this->setEtcData();

    }

    function setShopData() {

    	//ショップ名
        $this->lfText(125, 60, $this->shopInfo['shopName'], 8, 'B');

        //URL
        if ($this->shopInfo["orderScrapingParam"]["url"]) {
			$url = $this->shopInfo["orderScrapingParam"]["url"];
        } else {
			$url = $this->companyInfo["url"];
        }
        $this->lfText(125, 63, $url, 8);

        //会社名
        $this->lfText(125, 68, $this->companyInfo["name"], 8);

        //郵便番号
        $text = '〒 '.$this->companyInfo["zipcode"];
        $this->lfText(125, 71, $text, 8);

        //住所
        $text = $this->companyInfo["address"];
        $this->lfText(125, 74, $text, 8);

        //TEL・FAX
        $text = 'TEL: '.$this->companyInfo["phone"];
        //FAX番号が存在する場合、表示する
        if (strlen($this->companyInfo["fax"]) > 0) {
            $text .= '　FAX: '.$this->companyInfo["fax"];
        }
        $this->lfText(125, 80, $text, 8);

        //Email
        if ($this->shopInfo["orderScrapingParam"]["mail_address"]) {
        	$email = 'Email: '.$this->shopInfo["orderScrapingParam"]["mail_address"];
        } else if (strlen($this->companyInfo["email"]) > 0) {
            $email = 'Email: '.$this->companyInfo["email"];
        }

        if ($email) {
        	$this->lfText(125, 83, $email, 8);
        }

        //ロゴ画像
        $logo_file = HUNGLEAD_PLUGIN_DIR."utils/" . 'logo.jpg';
        $this->Image($logo_file, 124, 46, 40);
    }

    function setMessageData() {
        // メッセージ
        $this->lfText(27, 70, $this->arrData['msg1'], 8);  //メッセージ1
        $this->lfText(27, 74, $this->arrData['msg2'], 8);  //メッセージ2
        $this->lfText(27, 78, $this->arrData['msg3'], 8);  //メッセージ3
        $text = '作成日: '.$this->arrData['year'].'年'.$this->arrData['month'].'月'.$this->arrData['day'].'日';
        $this->lfText(158, 288, $text, 8);  //作成日
    }

    function setOrderData() {

    	// 購入者情報
        $text = '〒 '.$this->arrDisp['order_zipcode'];
        $this->lfText(23, 43, $text, 10); //注文者郵便番号
        $text = $this->arrDisp['order_address'];
        $this->lfText(27, 47, $text, 10); //注文者住所
        $text = $this->arrDisp['order_name'].'　様';
        $this->lfText(27, 59, $text, 11); //注文者氏名

        // お届け先情報
        $this->SetFont('SJIS', '', 10);
        $this->lfText(25, 125, SC_Utils_Ex::sfDispDBDate($this->arrDisp['order_date']), 10); //受注日時
        $this->lfText(25, 135, $this->arrDisp['order_code'], 10); //受注番号

        $this->SetFont('Gothic', 'B', 15);
        $this->Cell(0, 10, $this->lfConvSjis($this->tpl_title), 0, 2, 'C', 0, '');  //文書タイトル（納品書・請求書）
        $this->Cell(0, 66, '', 0, 2, 'R', 0, '');
        $this->Cell(5, 0, '', 0, 0, 'R', 0, '');
        $this->SetFont('SJIS', 'B', 15);
        $this->Cell(67, 8, $this->lfConvSjis(number_format($this->arrDisp['order_price']).' 円'), 0, 2, 'R', 0, '');
        $this->Cell(0, 45, '', 0, 2, '', 0, '');

        $this->SetFont('SJIS', '', 8);

        $monetary_unit = $this->lfConvSjis('円');
        $point_unit = $this->lfConvSjis('Pt');

        // 購入商品情報

        for ($i = 0; $i < count($this->arrDisp['details']); $i++) {

        	$detail = $this->arrDisp['details'][$i];

            $arrOrder[$i][0]  = $this->lfConvSjis($detail['item_name'].' / ');
            $arrOrder[$i][0] .= $this->lfConvSjis($detail['item_code'].' / ');
            if ($detail['item_select']) {
                $arrOrder[$i][0] .= $this->lfConvSjis($detail['item_select'].' / ');
            }
            $arrOrder[$i][1]  = number_format($detail["quantity"]);
            $arrOrder[$i][2]  = number_format($detail["unit_price"]).$monetary_unit;
            $arrOrder[$i][3]  = number_format($detail["item_subtotal"]).$monetary_unit;
        }

        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = '';
        $arrOrder[$i][3] = '';

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('商品合計');
        $arrOrder[$i][3] = number_format($this->arrDisp['subtotal']).$monetary_unit;

        if ($this->arrDisp['order_tax'] > 0) {
            $i++;
            $arrOrder[$i][0] = '';
            $arrOrder[$i][1] = '';
            $arrOrder[$i][2] = $this->lfConvSjis('消費税');
            $arrOrder[$i][3] = number_format($this->arrDisp['order_tax']).$monetary_unit;
        }

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('送料');
        $arrOrder[$i][3] = number_format($this->arrDisp['delivery_cost']).$monetary_unit;

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('手数料');
        $arrOrder[$i][3] = number_format($this->arrDisp['commission']).$monetary_unit;

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('包装料');
        $arrOrder[$i][3] = number_format($this->arrDisp['giftwrap_cost']).$monetary_unit;


        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('割引金額');
        $arrOrder[$i][3] = '- '.number_format($this->arrDisp['discount']).$monetary_unit;

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('利用ポイント');
        $arrOrder[$i][3] = '- '.number_format($this->arrDisp['use_point']).$monetary_unit;

        $i++;
        $arrOrder[$i][0] = '';
        $arrOrder[$i][1] = '';
        $arrOrder[$i][2] = $this->lfConvSjis('請求金額');
        $arrOrder[$i][3] = number_format($this->arrDisp['order_price']).$monetary_unit;

        $this->FancyTable($this->label_cell, $arrOrder, $this->width_cell);
    }

    function setEtcData() {
        $this->Cell(0, 10, '', 0, 1, 'C', 0, '');
        $this->SetFont('Gothic', 'B', 9);
        $this->MultiCell(0, 6, $this->lfConvSjis('＜ 備 考 ＞'), 'T', 2, 'L', 0, '');  //備考
        $this->Ln();
        $this->SetFont('SJIS', '', 8);
        $this->MultiCell(0, 4, $this->lfConvSjis($this->arrData['etc1']."\n".$this->arrData['etc2']."\n".$this->arrData['etc3']), '', 2, 'L', 0, '');  //備考
    }

    function createPdf() {
        // PDFをブラウザに送信
        ob_clean();
        if ($this->pdf_download == 1) {
            if ($this->PageNo() == 1) {
                $filename = 'nouhinsyo-No'.$this->arrData['order_id'].'.pdf';
            } else {
                $filename = 'nouhinsyo.pdf';
            }
            $this->Output($this->lfConvSjis($filename), 'D');
        } else {
            $this->Output();
        }

        // 入力してPDFファイルを閉じる
        $this->Close();
    }

    // PDF_Japanese::Text へのパーサー
    function lfText($x, $y, $text, $size = 0, $style = '') {
        // 退避
        $bak_font_style = $this->FontStyle;
        $bak_font_size = $this->FontSizePt;

        $this->SetFont('', $style, $size);
        $this->Text($x, $y, $this->lfConvSjis($text));

        // 復元
        $this->SetFont('', $bak_font_style, $bak_font_size);
    }

    // 受注データの取得
    function lfGetData($order_id) {
        if (SC_Utils_Ex::sfIsInt($order_id)) {

        	$order = ApiUtil::getOrderInfo($order_id);

        	foreach ($order["details"] as &$detail) {
        		$detail["item_subtotal"] = $detail["unit_price"] * $detail["quantity"];
        		$subtotal += $detail["item_subtotal"];
        	}

        	$order["subtotal"] = $subtotal;

        	$this->arrDisp = $order;

        	$this->shopInfo = ApiUtil::getShopInfo($order["shop_id"]);
        }
    }

    // 文字コードSJIS変換 -> japanese.phpで使用出来る文字コードはSJIS-winのみ
    function lfConvSjis($conv_str) {
        return mb_convert_encoding($conv_str, 'SJIS-win', CHAR_CODE);
    }

    /**
     * ゴシックフォントを定義
     *
     * @return void
     */
    function lfAddGothicFont() {
        $cw = $GLOBALS['SJIS_widths'];
        $c_map = '90msp-RKSJ-H';
        $registry = array('ordering'=>'Japan1','supplement'=>2);
        $this->AddCIDFonts('Gothic', 'KozGoPro-Medium-Acro,MS-PGothic,Osaka', $cw, $c_map, $registry);
    }

    /**
     * フッター
     *
     * 現状の PDF_Japanese#Footer の動作によって、生成される PDF がエラーとなるケースがあり、
     * そのエラーを抑える意図。
     * @return void
     */
    function Footer() {
    }
}
