<?php
/*
 * This file is part of Robot-in
 *
 * Copyright (C) 2006-2012 Hunglead,Inc. All Rights Reserved.
 *
 * http://www.hunglead.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

require_once HUNGLEAD_PLUGIN_DIR. "harc/ApiUtil.php";

/**
 *
 * メール送信用ヘルパークラス
 * @author hunglead
 */
class CustomMailHelper {

	/**
	 * 店舗情報キャッシュ用配列
	 * @var array
	 */
	private $shopInfoCache = array();

	/**
	 * メール情報を作成
	 * @param number $orderId 受注ID
	 * @param number $templateId テンプレートID
	 * @param string $subject タイトル
	 * @param string $header ヘッダー
	 * @param string $footer フッター
	 */
	public function createMailContent($orderId, $templateId, $subject, $header, $footer) {

		$arrTplVar = new stdClass();

		$objQuery = new SC_Query_Ex();

		if ($subject == '' && $header == '' && $footer == '') {
			// メールテンプレート情報の取得
			$where = 'template_id = ?';
			$arrRet = $objQuery->select('subject, header, footer', 'dtb_mailtemplate', $where, array($templateId));
			$arrTplVar->tpl_header = $arrRet[0]['header'];
			$arrTplVar->tpl_footer = $arrRet[0]['footer'];
			$tmp_subject = $arrRet[0]['subject'];
		} else {
			$arrTplVar->tpl_header = $header;
			$arrTplVar->tpl_footer = $footer;
			$tmp_subject = $subject;
		}

		$arrTplVar->order = $this->getOrder($orderId);

		$siteView = new SC_SiteView_Ex();
		$siteView->assignobj($arrTplVar);

		$result = array();

		$companyInfo = $this->getCompanyInfo();

		$result["from"] = $companyInfo["email"];
		$result["to"] = $arrTplVar->order["order_email"];
		$result["subject"] = $tmp_subject;
		$result["body"] = $siteView->fetch(HUNGLEAD_PLUGIN_DIR."template/admin/multistore/mail/order_mail.tpl");

		$shopInfo = $this->getShopInfo($arrTplVar->order["shop_id"]);

		if ($shopInfo["orderScrapingParam"]["mail_address"]) {
			$result["from"] = $shopInfo["orderScrapingParam"]["mail_address"];
		}

		if ($shopInfo["orderScrapingParam"]["mail_host"]) {
			$result["host"] = $shopInfo["orderScrapingParam"]["mail_host"];
		}

		if ($shopInfo["orderScrapingParam"]["mail_port"]) {
			$result["port"] = $shopInfo["orderScrapingParam"]["mail_port"];
		}

		if ($shopInfo["orderScrapingParam"]["mail_smtp_user"]) {

			$result["auth"] = true;
			$result["username"] = $shopInfo["orderScrapingParam"]["mail_smtp_user"];
			$result["password"] = $shopInfo["orderScrapingParam"]["mail_smtp_password"];
		}

		return $result;
	}

	/**
	 * 会社情報を取得
	 * @return array 会社情報
	 */
	private function getCompanyInfo() {
		if ($this->companyInfo) {
			return $this->companyInfo;
		}

		$companyInfo = ApiUtil::getCompanyInfo();
		$this->companyInfo = $companyInfo;

		return $companyInfo;
	}

	/**
	 * 店舗情報を取得
	 * @param number $shopId 店舗ID
	 * @return array 店舗情報
	 */
	private function getShopInfo($shopId) {

		if ($this->shopInfoCache[$shopId]) {
			return $this->shopInfoCache[$shopId];
		}

		$shopInfo = ApiUtil::getShopInfo($shopId);
		$this->shopInfoCache[$shopId] = $shopInfo;

		return $shopInfo;
	}

	/**
	 * 受注情報を取得
	 * @param number $orderId 受注ID
	 * @return array 受注情報
	 */
	private function getOrder($orderId) {

		$order = ApiUtil::getOrderInfo($orderId);

		foreach ($order["details"] as &$detail) {
			$detail["item_subtotal"] = $detail["unit_price"] * $detail["quantity"];
			$subtotal += $detail["item_subtotal"];
		}

		$order["subtotal"] = $subtotal;

		return $order;
	}

	/**
	 * メールを送信
	 * @param array $content メール情報
	 */
	public function sendMail($content) {

		if ($content["host"]) {
			$driver = "smtp";
			$param = array();
			$param["host"] = $content["host"];

			if ($content["port"]) {
				$param["port"] = $content["port"];
			} else {
				$param["port"] = 25;
			}

			if ($content["auth"]) {
				$param["auth"] = $content["auth"];
			}

			if ($content["username"]) {
				$param["username"] = $content["username"];
			}

			if ($content["password"]) {
				$param["password"] = $content["password"];
			}

		} else {
			$driver = MAIL_BACKEND;
			$param = $this->getBackendParams(MAIL_BACKEND);
		}

		$sender =& Mail::factory($driver, $param);

		$recipients = array();
		$recipients[] = $content["to"];
		$header = $this->getTEXTHeader($content);
		$body = mb_convert_encoding($content["body"], 'JIS', CHAR_CODE);

		$result = $sender->send($recipients, $header, $body);

		if (PEAR::isError($result)) {
			GC_Utils_Ex::gfPrintLog($result->getMessage());
			GC_Utils_Ex::gfDebugLog($header);
			return false;
		}

		return true;
	}

    // ヘッダーを返す
    function getBaseHeader($content) {
        //-- 送信するメールの内容と送信先
        $arrHeader['MIME-Version'] = '1.0';
        $subject = $content["subject"];
        $subject = mb_encode_mimeheader($subject, 'JIS', 'B', "\n");
        $subject = str_replace("\x0D\x0A", "\n", $subject);
        $subject = str_replace("\x0D", "\n", $subject);
        $subject = str_replace("\x0A", "\n", $subject);
        $arrHeader['Subject'] = $subject;
        $arrHeader['From'] = $content["from"];
        $arrHeader['To'] = $content["to"];

        if ($content["return-path"]) {
        	$arrHeader['Return-Path'] = $content["return-path"];
        } else {
        	$arrHeader['Return-Path'] = $content["from"];
        }

        if ($content["reply-to"] != '') {
            $arrHeader['Reply-To'] = $this->reply_to;
        }
        if ($content["cc"] != '') {
            $arrHeader['Cc'] = $this->cc;
        }
        if ($content["bcc"] != '') {
            $arrHeader['Bcc'] = $this->bcc;
        }

        $arrHeader['Date'] = date('D, j M Y H:i:s O');
        return $arrHeader;
    }

    // ヘッダーを返す
    private function getTEXTHeader($content) {
        $arrHeader = $this->getBaseHeader($content);
        $arrHeader['Content-Type'] = "text/plain; charset=\"ISO-2022-JP\"";
        $arrHeader['Content-Transfer-Encoding'] = '7bit';
        return $arrHeader;
    }

    // ヘッダーを返す
//     private function getHTMLHeader() {
//         $arrHeader = $this->getBaseHeader();
//         $arrHeader['Content-Type'] = "text/html; charset=\"ISO-2022-JP\"";
//         $arrHeader['Content-Transfer-Encoding'] = 'ISO-2022-JP';
//         return $arrHeader;
//     }


    /**
    * メーラーバックエンドに応じたパラメーターを返す.
    *
    * @param string $backend Pear::Mail のバックエンド
    * @return array メーラーバックエンドに応じたパラメーターの配列
    */
    private function getBackendParams($backend) {
    	switch ($backend) {
    		case 'mail':
    			$arrParams = array();
    			break;
            case 'sendmail':
    			$arrParams = array(
    				'sendmail_path' => '/usr/bin/sendmail',
    				'sendmail_args' => '-i',
    			);
    			break;
    		case 'smtp':
    		default:
                $arrParams = array(
                    'host' => $this->host,
                    'port' => $this->port,
                );
    			break;
    	}
		return $arrParams;
	}
}